@@ -1,17 +1,19 @@
 //! Corrections that can be performed on visibility data
 
+use crate::{
+    ndarray::{parallel::prelude::*, Array3, Axis},
+    Jones,
+};
 use indicatif::{ProgressBar, ProgressStyle};
 use log::trace;
-use mwa_rust_core::{
-    constants::VEL_C, mwalib::CorrelatorContext, precession::precess_time, time, Complex, Jones,
+use marlu::{
+    constants::VEL_C, mwalib::CorrelatorContext, precession::precess_time, time, Complex,
     LatLngHeight, RADec, XyzGeodetic, UVW,
 };
 use std::{f64::consts::PI, ops::Range};
 
-use ndarray::{parallel::prelude::*, Array3, Axis};
-
 /// Perform cable length corrections, given an observation's
-/// [`mwalib::CorrelatorContext`] and an ['ndarray::Array3`] of [`Jones`]
+/// [`mwalib::CorrelatorContext`] and an ['ndarray::Array3`] of [`TestJones`]
 /// visibilities
 ///
 /// Cable lengths are determined by the difference between a baseline's rfInput
@@ -145,7 +147,7 @@ pub fn correct_cable_lengths(
 }
 
 /// Perform geometric corrections, given an observation's
-/// [`mwalib::CorrelatorContext`] and an ['ndarray::Array3`] of [`Jones`]
+/// [`mwalib::CorrelatorContext`] and an ['ndarray::Array3`] of [`TestJones`]
 /// visibilities
 ///
 /// Complex visibilities are phase-shifted by an angle determined by the length
@@ -287,14 +289,13 @@ mod tests {
     #![allow(clippy::float_cmp)]
 
     use super::{correct_cable_lengths, correct_geometry, VEL_C};
-    use approx::assert_abs_diff_eq;
-    use mwa_rust_core::{
-        mwalib::CorrelatorContext, precession::precess_time, time, Complex, Jones, LatLngHeight,
-        RADec, XyzGeodetic, UVW,
+    use marlu::{
+        approx::assert_abs_diff_eq, mwalib::CorrelatorContext, precession::precess_time, time,
+        Complex, Jones, LatLngHeight, RADec, XyzGeodetic, UVW,
     };
     use std::f64::consts::PI;
 
-    use crate::{context_to_jones_array, get_flaggable_timesteps};
+    use crate::{context_to_jones_array, get_flaggable_timesteps, TestJones};
 
     // TODO: Why does clippy think CxxImageSet.ImageBuffer() is &[f64]?
     // TODO: deduplicate from lib.rs
@@ -336,7 +337,7 @@ mod tests {
 
         // let img_baseline_idxs: Vec<usize> = (0..context.metafits_context.num_baselines).collect();
 
-        let (mut jones_array, _) = context_to_jones_array(
+        let (jones_array, _) = context_to_jones_array(
             &context,
             &img_timestep_range,
             &img_coarse_chan_range,
@@ -344,11 +345,13 @@ mod tests {
             None,
         );
 
+        let jones_array = jones_array.mapv(TestJones::from);
+
         // ts 0, chan 0 (cc 0, fc 0), baseline 0
         let viz_0_0_0 = *jones_array.get((0, 0, 0)).unwrap();
         assert_abs_diff_eq!(
             viz_0_0_0,
-            Jones::from([
+            TestJones::from([
                 Complex::new(0x410000 as f32, 0x410001 as f32),
                 Complex::new(0x410002 as f32, 0x410003 as f32),
                 Complex::new(0x410004 as f32, 0x410005 as f32),
@@ -360,7 +363,7 @@ mod tests {
         let viz_0_0_1 = *jones_array.get((0, 0, 1)).unwrap();
         assert_abs_diff_eq!(
             viz_0_0_1,
-            Jones::from([
+            TestJones::from([
                 Complex::new(0x410010 as f32, 0x410011 as f32),
                 Complex::new(0x410012 as f32, 0x410013 as f32),
                 Complex::new(0x410014 as f32, 0x410015 as f32),
@@ -372,7 +375,7 @@ mod tests {
         let viz_3_3_1 = *jones_array.get((3, 3, 1)).unwrap();
         assert_abs_diff_eq!(
             viz_3_3_1,
-            Jones::from([
+            TestJones::from([
                 Complex::new(0x410718 as f32, 0x410719 as f32),
                 Complex::new(0x41071a as f32, 0x41071b as f32),
                 Complex::new(0x41071c as f32, 0x41071d as f32),
@@ -439,8 +442,12 @@ mod tests {
         let (sin_1_yy_3_f64, cos_1_yy_3_f64) = angle_1_yy_3.sin_cos();
         let (sin_1_yy_3, cos_1_yy_3) = (sin_1_yy_3_f64 as f32, cos_1_yy_3_f64 as f32);
 
+        // TODO: this is not great.
+        let mut jones_array = jones_array.mapv(|e| Jones::from([e[0], e[1], e[2], e[3]]));
         correct_cable_lengths(&context, &mut jones_array, &img_coarse_chan_range);
 
+        let jones_array = jones_array.mapv(TestJones::from);
+
         // there should be no difference in baseline 0
         // ts 0, chan 0, baseline 0
         assert_abs_diff_eq!(*jones_array.get((0, 0, 0)).unwrap(), viz_0_0_0);
@@ -464,7 +471,7 @@ mod tests {
 
         assert_abs_diff_eq!(
             *jones_array.get((0, 0, 1)).unwrap(),
-            &Jones::from([
+            &TestJones::from([
                 Complex::new(rot_1_xx_0_0_re, rot_1_xx_0_0_im),
                 Complex::new(rot_1_xy_0_0_re, rot_1_xy_0_0_im),
                 Complex::new(rot_1_yx_0_0_re, rot_1_yx_0_0_im),
@@ -488,7 +495,7 @@ mod tests {
 
         assert_abs_diff_eq!(
             *jones_array.get((3, 3, 1)).unwrap(),
-            &Jones::from([
+            &TestJones::from([
                 Complex::new(rot_1_xx_3_3_re, rot_1_xx_3_3_im),
                 Complex::new(rot_1_xy_3_3_re, rot_1_xy_3_3_im),
                 Complex::new(rot_1_yx_3_3_re, rot_1_yx_3_3_im),
@@ -513,7 +520,7 @@ mod tests {
 
         // let img_baseline_idxs: Vec<usize> = (0..context.metafits_context.num_baselines).collect();
 
-        let (mut jones_array, _) = context_to_jones_array(
+        let (jones_array, _) = context_to_jones_array(
             &context,
             &img_timestep_range,
             &img_coarse_chan_range,
@@ -521,11 +528,13 @@ mod tests {
             None,
         );
 
+        let jones_array = jones_array.mapv(TestJones::from);
+
         // ts 0, chan 0 (cc 0, fc 0), baseline 0
         let viz_0_0_0 = *jones_array.get((0, 0, 0)).unwrap();
         assert_abs_diff_eq!(
             viz_0_0_0,
-            Jones::from([
+            TestJones::from([
                 Complex::new(0x10c5be as f32, -0x10c5bf as f32),
                 Complex::new(0x10c5ae as f32, 0x10c5af as f32),
                 Complex::new(0x10c5ae as f32, -0x10c5af as f32),
@@ -537,7 +546,7 @@ mod tests {
         let viz_0_0_5 = *jones_array.get((0, 0, 5)).unwrap();
         assert_abs_diff_eq!(
             viz_0_0_5,
-            Jones::from([
+            TestJones::from([
                 Complex::new(0x10f1ce as f32, -0x10f1cf as f32),
                 Complex::new(0x10ea26 as f32, -0x10ea27 as f32),
                 Complex::new(0x10f1be as f32, -0x10f1bf as f32),
@@ -549,7 +558,7 @@ mod tests {
         let viz_3_3_5 = *jones_array.get((3, 3, 5)).unwrap();
         assert_abs_diff_eq!(
             viz_3_3_5,
-            Jones::from([
+            TestJones::from([
                 Complex::new(0x0df3ce as f32, -0x0df3cf as f32),
                 Complex::new(0x0dec26 as f32, -0x0dec27 as f32),
                 Complex::new(0x0df3be as f32, -0x0df3bf as f32),
@@ -616,8 +625,13 @@ mod tests {
         let (sin_5_yy_3_f64, cos_5_yy_3_f64) = angle_5_yy_3.sin_cos();
         let (sin_5_yy_3, cos_5_yy_3) = (sin_5_yy_3_f64 as f32, cos_5_yy_3_f64 as f32);
 
+        // FIXME: aaaaaaa
+        let mut jones_array = jones_array.mapv(|e| Jones::from([e[0], e[1], e[2], e[3]]));
+
         correct_cable_lengths(&context, &mut jones_array, &img_coarse_chan_range);
 
+        let jones_array = jones_array.mapv(TestJones::from);
+
         // there should be no difference in baseline 0
         // ts 0 (batch 0, scan 0), chan 0 (cc 0, fc 0), baseline 0
         assert_abs_diff_eq!(*jones_array.get((0, 0, 0)).unwrap(), viz_0_0_0);
@@ -641,7 +655,7 @@ mod tests {
 
         assert_abs_diff_eq!(
             *jones_array.get((0, 0, 5)).unwrap(),
-            &Jones::from([
+            &TestJones::from([
                 Complex::new(rot_5_xx_0_0_re, rot_5_xx_0_0_im),
                 Complex::new(rot_5_xy_0_0_re, rot_5_xy_0_0_im),
                 Complex::new(rot_5_yx_0_0_re, rot_5_yx_0_0_im),
@@ -665,7 +679,7 @@ mod tests {
 
         assert_abs_diff_eq!(
             *jones_array.get((3, 3, 5)).unwrap(),
-            &Jones::from([
+            &TestJones::from([
                 Complex::new(rot_5_xx_3_3_re, rot_5_xx_3_3_im),
                 Complex::new(rot_5_xy_3_3_re, rot_5_xy_3_3_im),
                 Complex::new(rot_5_yx_3_3_re, rot_5_yx_3_3_im),
@@ -695,7 +709,7 @@ mod tests {
         // let phase_centre_ha = phase_centre_ra.to_hadec(lst_rad);
         let tiles_xyz_geod = XyzGeodetic::get_tiles_mwa(&context.metafits_context);
 
-        let (mut jones_array, _) = context_to_jones_array(
+        let (jones_array, _) = context_to_jones_array(
             &context,
             &img_timestep_range,
             &img_coarse_chan_range,
@@ -703,11 +717,13 @@ mod tests {
             None,
         );
 
+        let jones_array = jones_array.mapv(TestJones::from);
+
         // ts 0, chan 0 (cc 0, fc 0), baseline 0
         let viz_0_0_0 = *jones_array.get((0, 0, 0)).unwrap();
         assert_abs_diff_eq!(
             viz_0_0_0,
-            Jones::from([
+            TestJones::from([
                 Complex::new(0x10c5be as f32, -0x10c5bf as f32),
                 Complex::new(0x10c5ae as f32, 0x10c5af as f32),
                 Complex::new(0x10c5ae as f32, -0x10c5af as f32),
@@ -719,7 +735,7 @@ mod tests {
         let viz_0_0_5 = *jones_array.get((0, 0, 5)).unwrap();
         assert_abs_diff_eq!(
             viz_0_0_5,
-            Jones::from([
+            TestJones::from([
                 Complex::new(0x10f1ce as f32, -0x10f1cf as f32),
                 Complex::new(0x10ea26 as f32, -0x10ea27 as f32),
                 Complex::new(0x10f1be as f32, -0x10f1bf as f32),
@@ -731,7 +747,7 @@ mod tests {
         let viz_3_3_5 = *jones_array.get((3, 3, 5)).unwrap();
         assert_abs_diff_eq!(
             viz_3_3_5,
-            Jones::from([
+            TestJones::from([
                 Complex::new(0x0df3ce as f32, -0x0df3cf as f32),
                 Complex::new(0x0dec26 as f32, -0x0dec27 as f32),
                 Complex::new(0x0df3be as f32, -0x0df3bf as f32),
@@ -790,6 +806,8 @@ mod tests {
 
         // let angle_5_3 = -2.0 * PI * w_5_3 * (all_freqs_hz[3] as f64) / VEL_C;
 
+        let mut jones_array = jones_array.mapv(|e| Jones::from([e[0], e[1], e[2], e[3]]));
+
         correct_geometry(
             &context,
             &mut jones_array,
@@ -798,6 +816,8 @@ mod tests {
             None,
         );
 
+        let jones_array = jones_array.mapv(TestJones::from);
+
         // there should be no difference in baseline 0
         // ts 0 (batch 0, scan 0), chan 0 (cc 0, fc 0), baseline 0
         assert_abs_diff_eq!(*jones_array.get((0, 0, 0)).unwrap(), viz_0_0_0);
@@ -821,7 +841,7 @@ mod tests {
 
         assert_abs_diff_eq!(
             *jones_array.get((0, 0, 5)).unwrap(),
-            &Jones::from([
+            &TestJones::from([
                 Complex::new(rot_5_xx_0_0_re, rot_5_xx_0_0_im),
                 Complex::new(rot_5_xy_0_0_re, rot_5_xy_0_0_im),
                 Complex::new(rot_5_yx_0_0_re, rot_5_yx_0_0_im),
@@ -845,7 +865,7 @@ mod tests {
 
         assert_abs_diff_eq!(
             *jones_array.get((3, 3, 5)).unwrap(),
-            &Jones::from([
+            &TestJones::from([
                 Complex::new(rot_5_xx_3_3_re, rot_5_xx_3_3_im),
                 Complex::new(rot_5_xy_3_3_re, rot_5_xy_3_3_im),
                 Complex::new(rot_5_yx_3_3_re, rot_5_yx_3_3_im),
@@ -874,7 +894,7 @@ mod tests {
         // let phase_centre_ha = phase_centre_ra.to_hadec(lst_rad);
         let tiles_xyz_geod = XyzGeodetic::get_tiles_mwa(&context.metafits_context);
 
-        let (mut jones_array, _) = context_to_jones_array(
+        let (jones_array, _) = context_to_jones_array(
             &context,
             &img_timestep_range,
             &img_coarse_chan_range,
@@ -882,11 +902,13 @@ mod tests {
             None,
         );
 
+        let jones_array = jones_array.mapv(TestJones::from);
+
         // ts 0, chan 0 (cc 0, fc 0), baseline 0
         let viz_0_0_0 = *jones_array.get((0, 0, 0)).unwrap();
         assert_abs_diff_eq!(
             viz_0_0_0,
-            Jones::from([
+            TestJones::from([
                 Complex::new(0x410000 as f32, 0x410001 as f32),
                 Complex::new(0x410002 as f32, 0x410003 as f32),
                 Complex::new(0x410004 as f32, 0x410005 as f32),
@@ -898,7 +920,7 @@ mod tests {
         let viz_0_0_1 = *jones_array.get((0, 0, 1)).unwrap();
         assert_abs_diff_eq!(
             viz_0_0_1,
-            Jones::from([
+            TestJones::from([
                 Complex::new(0x410010 as f32, 0x410011 as f32),
                 Complex::new(0x410012 as f32, 0x410013 as f32),
                 Complex::new(0x410014 as f32, 0x410015 as f32),
@@ -910,7 +932,7 @@ mod tests {
         let viz_3_3_1 = *jones_array.get((3, 3, 1)).unwrap();
         assert_abs_diff_eq!(
             viz_3_3_1,
-            Jones::from([
+            TestJones::from([
                 Complex::new(0x410718 as f32, 0x410719 as f32),
                 Complex::new(0x41071a as f32, 0x41071b as f32),
                 Complex::new(0x41071c as f32, 0x41071d as f32),
@@ -968,7 +990,7 @@ mod tests {
         let (sin_3_3_1, cos_3_3_1) = (sin_3_3_1_f64 as f32, cos_3_3_1_f64 as f32);
 
         // let angle_5_3 = -2.0 * PI * w_5_3 * (all_freqs_hz[3] as f64) / VEL_C;
-
+        let mut jones_array = jones_array.mapv(|e| Jones::from([e[0], e[1], e[2], e[3]]));
         correct_geometry(
             &context,
             &mut jones_array,
@@ -976,7 +998,7 @@ mod tests {
             &img_coarse_chan_range,
             None,
         );
-
+        let jones_array = jones_array.mapv(TestJones::from);
         // there should be no difference in baseline 0
         // ts 0 (batch 0, scan 0), chan 0 (cc 0, fc 0), baseline 0
         assert_abs_diff_eq!(*jones_array.get((0, 0, 0)).unwrap(), viz_0_0_0);
@@ -1000,7 +1022,7 @@ mod tests {
 
         assert_abs_diff_eq!(
             *jones_array.get((0, 0, 1)).unwrap(),
-            &Jones::from([
+            &TestJones::from([
                 Complex::new(rot_1_xx_0_0_re, rot_1_xx_0_0_im),
                 Complex::new(rot_1_xy_0_0_re, rot_1_xy_0_0_im),
                 Complex::new(rot_1_yx_0_0_re, rot_1_yx_0_0_im),
@@ -1024,7 +1046,7 @@ mod tests {
 
         assert_abs_diff_eq!(
             *jones_array.get((3, 3, 1)).unwrap(),
-            &Jones::from([
+            &TestJones::from([
                 Complex::new(rot_1_xx_3_3_re, rot_1_xx_3_3_im),
                 Complex::new(rot_1_xy_3_3_re, rot_1_xy_3_3_im),
                 Complex::new(rot_1_yx_3_3_re, rot_1_yx_3_3_im),
