@@ -8,12 +8,12 @@ use crate::{
         BirliError::{NoCommonTimesteps, NoProvidedTimesteps},
     },
     io::error::IOError,
+    ndarray::{Array2, Array3, ArrayView3, Axis, Zip},
     FlagFileSet,
 };
 use cfg_if::cfg_if;
 use log::trace;
-use mwa_rust_core::mwalib::CorrelatorContext;
-use ndarray::{Array2, Array3, ArrayView3, Axis, Zip};
+use marlu::mwalib::CorrelatorContext;
 use rayon::prelude::*;
 
 cfg_if! {
@@ -24,7 +24,7 @@ cfg_if! {
         use aoflagger_sys::{CxxAOFlagger, CxxFlagMask, UniquePtr, flagmask_or,
             flagmask_set};
         use indicatif::{ProgressBar, ProgressStyle};
-        use mwa_rust_core::Jones;
+        use marlu::Jones;
     }
 }
 
@@ -81,11 +81,11 @@ pub fn get_flaggable_timesteps(context: &CorrelatorContext) -> Result<Vec<usize>
         context.common_timestep_indices.first(),
         context.provided_timestep_indices.last(),
     ) {
-        (Some(&first), Some(&last)) => Ok((first..last + 1).collect()),
+        (Some(&first), Some(&last)) if first <= last => Ok((first..last + 1).collect()),
         (.., None) => Err(NoProvidedTimesteps {
             timestep_info: format!("{:?}", &context.gpubox_time_map),
         }),
-        (None, ..) => Err(NoCommonTimesteps {
+        _ => Err(NoCommonTimesteps {
             timestep_info: format!("{:?}", &context.gpubox_time_map),
         }),
     }
@@ -460,7 +460,7 @@ mod tests {
 
     use crate::{
         error::BirliError::{NoCommonTimesteps, NoProvidedTimesteps},
-        mwa_rust_core::mwalib::CorrelatorContext,
+        marlu::mwalib::CorrelatorContext,
         FlagFileSet,
     };
 
@@ -652,22 +652,26 @@ mod tests {
     }
 }
 
+/// Get the weight factor of an observation's context.
+///
+/// This is a conceptfrom Cotter, and the legacy MWA correlator where the value
+/// is a multiple of the frequency averaging factor (relative to 10kHz), and the
+/// time averaging factor (relative to 1s).
+pub fn get_weight_factor(context: &CorrelatorContext) -> f64 {
+    let integration_time_s = context.metafits_context.corr_int_time_ms as f64 / 1000.0;
+    let fine_chan_width_hz = context.metafits_context.corr_fine_chan_width_hz as f64;
+    fine_chan_width_hz * integration_time_s / 10000.0
+}
+
 /// Convert the given ndarray of boolean flags to an ndarray of float weights
 /// TODO: deal with weight factor when doing averaging.
 /// TODO: deal with passband gains
-pub fn flag_to_weight_array(
-    context: &CorrelatorContext,
-    flag_array: ArrayView3<bool>,
-) -> Array3<f32> {
-    let integration_time_s = context.metafits_context.corr_int_time_ms as f64 / 1000.0;
-    let fine_chan_width_hz = context.metafits_context.corr_fine_chan_width_hz as f64;
-    let weight_factor = fine_chan_width_hz * integration_time_s / 10000.0;
-
+pub fn flag_to_weight_array(flag_array: ArrayView3<bool>, weight_factor: f64) -> Array3<f32> {
     let mut weight_array = Array3::from_elem(flag_array.dim(), 0.0_f32);
 
     Zip::from(&mut weight_array)
         .and(&flag_array)
-        .par_apply(|weight, &flag| {
+        .par_for_each(|weight, &flag| {
             *weight = if flag { -weight_factor } else { weight_factor } as f32;
         });
 
@@ -679,7 +683,7 @@ pub fn flag_to_weight_array(
 /// Tests which require the use of the aoflagger feature
 mod tests_aoflagger {
     use super::{get_flaggable_timesteps, init_flag_array};
-    use mwa_rust_core::{mwalib::CorrelatorContext, Complex, Jones};
+    use marlu::{mwalib::CorrelatorContext, Complex, Jones};
     use ndarray::Array3;
 
     use crate::{
