@@ -9,10 +9,18 @@ pub mod uvfits;
 use std::{ops::Range, path::Path};
 
 use log::trace;
-use mwa_rust_core::{mwalib::CorrelatorContext, Jones, LatLngHeight};
-use ndarray::{Array3, ArrayView3, ArrayViewMut3};
+use marlu::{
+    io::{ms::MeasurementSetWriter, VisWritable},
+    mwalib::CorrelatorContext,
+    Jones, LatLngHeight, RADec,
+};
 use uvfits::UvfitsWriter;
 
+use crate::{
+    flags::get_weight_factor,
+    ndarray::{Array3, Array4, ArrayView3, ArrayViewMut3},
+};
+
 use self::error::IOError;
 
 /// The container has visibilities which can be read by passing in a mwalib
@@ -75,7 +83,7 @@ pub trait WriteableVis: Sync + Send {
 ///     get_flaggable_timesteps,
 ///     context_to_jones_array,
 ///     write_uvfits,
-///     mwa_rust_core::mwalib::CorrelatorContext
+///     marlu::mwalib::CorrelatorContext
 /// };
 ///
 /// // define our input files
@@ -132,8 +140,8 @@ pub trait WriteableVis: Sync + Send {
 ///
 /// TODO: replace all these args with birli_context
 #[allow(clippy::too_many_arguments)]
-pub fn write_uvfits<'a>(
-    filename: &'a Path,
+pub fn write_uvfits<T: AsRef<Path>>(
+    path: T,
     context: &CorrelatorContext,
     jones_array: &Array3<Jones<f32>>,
     flag_array: &Array3<bool>,
@@ -142,10 +150,10 @@ pub fn write_uvfits<'a>(
     mwalib_baseline_idxs: &[usize],
     array_pos: Option<LatLngHeight>,
 ) -> Result<(), IOError> {
-    trace!("start write_uvfits to {:?}", filename);
+    trace!("start write_uvfits to {:?}", path.as_ref());
 
     let mut uvfits_writer = UvfitsWriter::from_mwalib(
-        filename,
+        path,
         context,
         mwalib_timestep_range,
         mwalib_coarse_chan_range,
@@ -169,6 +177,131 @@ pub fn write_uvfits<'a>(
     Ok(())
 }
 
+/// Write the given ndarrays of flags and [`Jones`] matrix visibilities to a
+/// measurement set.
+///
+/// mwalib timestep, coarse channel and baseline indices are needed to map between
+/// indices in the arrays and indices according to mwalib, which are not the same.
+///
+/// # Examples
+///
+/// ```rust
+/// use tempfile::tempdir;
+/// use birli::{
+///     get_flaggable_timesteps,
+///     context_to_jones_array,
+///     write_ms,
+///     marlu::mwalib::CorrelatorContext
+/// };
+///
+/// // define our input files
+/// let metafits_path = "tests/data/1196175296_mwa_ord/1196175296.metafits";
+/// let gpufits_paths = vec![
+///     "tests/data/1196175296_mwa_ord/1196175296_20171201145440_gpubox01_00.fits",
+/// ];
+///
+/// // define a temporary directory for output files
+/// let tmp_dir = tempdir().unwrap();
+///
+/// // define our output flag file template
+/// let ms_out = tmp_dir.path().join("synthetic.ms");
+///
+/// // Create an mwalib::CorrelatorContext for accessing visibilities.
+/// let context = CorrelatorContext::new(&metafits_path, &gpufits_paths).unwrap();
+///
+/// // Determine which timesteps and coarse channels we want to use
+/// let img_timestep_idxs = get_flaggable_timesteps(&context).unwrap();
+/// let img_timestep_range =
+///     *img_timestep_idxs.first().unwrap()..(*img_timestep_idxs.last().unwrap() + 1);
+/// let img_coarse_chan_idxs = &context.common_coarse_chan_indices;
+/// let img_coarse_chan_range =
+///     *img_coarse_chan_idxs.first().unwrap()..(*img_coarse_chan_idxs.last().unwrap() + 1);
+///
+///
+/// // generate an array of jones matrices
+/// let (jones_array, flag_array) = context_to_jones_array(
+///     &context,
+///     &img_timestep_range,
+///     &img_coarse_chan_range,
+///     None,
+/// );
+///
+/// // write the visibilities to disk as .ms
+///
+/// let baseline_idxs = (0..context.metafits_context.num_baselines).collect::<Vec<_>>();
+///
+/// write_ms(
+///     ms_out.as_path(),
+///     &context,
+///     &jones_array,
+///     &flag_array,
+///     &img_timestep_range,
+///     &img_coarse_chan_range,
+///     &baseline_idxs,
+///     None,
+/// )
+/// .unwrap();
+/// ```
+/// # Errors
+///
+/// See: [`UvfitsWriter`]
+///
+/// TODO: replace all these args with birli_context
+#[allow(clippy::too_many_arguments)]
+pub fn write_ms<T: AsRef<Path>>(
+    path: T,
+    context: &CorrelatorContext,
+    jones_array: &Array3<Jones<f32>>,
+    flag_array: &Array3<bool>,
+    mwalib_timestep_range: &Range<usize>,
+    mwalib_coarse_chan_range: &Range<usize>,
+    mwalib_baseline_idxs: &[usize],
+    array_pos: Option<LatLngHeight>,
+) -> Result<(), IOError> {
+    trace!("start write_ms to {:?}", path.as_ref());
+
+    let phase_centre = RADec::from_mwalib_phase_or_pointing(&context.metafits_context);
+    let mut ms_writer = MeasurementSetWriter::new(path, phase_centre, array_pos);
+
+    ms_writer
+        .initialize_from_mwalib(context, mwalib_timestep_range, mwalib_coarse_chan_range)
+        .unwrap();
+
+    let jones_shape = jones_array.shape();
+    let flag_shape = flag_array.shape();
+    let num_pols = context.metafits_context.num_visibility_pols;
+    assert_eq!(jones_shape[0], flag_shape[0]);
+    assert_eq!(jones_shape[1], flag_shape[1]);
+    assert_eq!(jones_shape[2], flag_shape[2]);
+
+    // TODO: this will change with averaging.
+    let weight_factor = get_weight_factor(context) as f32;
+    let weight_array = Array4::from_shape_fn(
+        (flag_shape[0], flag_shape[1], flag_shape[2], num_pols),
+        |_| weight_factor,
+    );
+    let flag_array_expanded = Array4::from_shape_fn(
+        (flag_shape[0], flag_shape[1], flag_shape[2], num_pols),
+        |(t, f, b, _)| *flag_array.get((t, f, b)).unwrap(),
+    );
+
+    ms_writer
+        .write_vis_mwalib(
+            jones_array.view(),
+            weight_array.view(),
+            flag_array_expanded.view(),
+            context,
+            mwalib_timestep_range,
+            mwalib_coarse_chan_range,
+            mwalib_baseline_idxs,
+        )
+        .unwrap();
+
+    trace!("end write_ms");
+
+    Ok(())
+}
+
 #[cfg(test)]
 #[cfg(feature = "aoflagger")]
 /// Tests which require the use of the aoflagger feature
@@ -181,7 +314,7 @@ mod tests_aoflagger {
     use fitsio::errors::check_status as fits_check_status;
     use float_cmp::{approx_eq, F32Margin};
     use itertools::izip;
-    use mwa_rust_core::{
+    use marlu::{
         fitsio, fitsio_sys,
         mwalib::{
             _get_required_fits_key, _open_fits, _open_hdu, fits_open, fits_open_hdu,
