@@ -12,21 +12,25 @@ use super::error::{
     IOError,
     IOError::{FitsIO, FitsOpen, InvalidFlagFilenameTemplate, InvalidGpuBox, MwafInconsistent},
 };
-use crate::error::BirliError;
+use crate::{
+    error::BirliError,
+    ndarray::{Array3, Axis},
+};
 use clap::crate_version;
-use fitsio::hdu::FitsHdu;
-use fitsio::tables::{ColumnDataDescription, ColumnDataType, ConcreteColumnDescription};
-use fitsio::FitsFile;
+use fitsio::{
+    hdu::FitsHdu,
+    tables::{ColumnDataDescription, ColumnDataType, ConcreteColumnDescription},
+    FitsFile,
+};
 use indicatif::{MultiProgress, ProgressBar, ProgressDrawTarget, ProgressStyle};
 use itertools::izip;
-use mwa_rust_core::{
+use marlu::{
     fitsio, fitsio_sys,
     mwalib::{
         CorrelatorContext, MWAVersion, _get_required_fits_key, _open_hdu, fits_open_hdu,
         get_required_fits_key,
     },
 };
-use ndarray::{Array3, Axis};
 // use rayon::prelude::*;
 use regex::Regex;
 use std::collections::BTreeMap;
@@ -477,7 +481,7 @@ mod tests {
     use crate::io::error::IOError::{FitsOpen, InvalidFlagFilenameTemplate};
     use crate::{get_flaggable_timesteps, init_flag_array};
     use fitsio::FitsFile;
-    use mwa_rust_core::{
+    use marlu::{
         fitsio,
         mwalib::{
             CorrelatorContext, _get_optional_fits_key, _open_hdu, fits_open_hdu,
@@ -667,8 +671,8 @@ mod tests {
         )
         .unwrap();
 
-        for (&gpubox_id, mut fptr) in flag_file_set.gpubox_fptrs.iter_mut() {
-            let header = FlagFileSet::read_header(&mut fptr).unwrap();
+        for (&gpubox_id, fptr) in flag_file_set.gpubox_fptrs.iter_mut() {
+            let header = FlagFileSet::read_header(fptr).unwrap();
             assert_eq!(header.obs_id, 1247842824);
             assert_eq!(header.num_channels, 128);
             assert_eq!(header.num_ants, 128);
