@@ -5,15 +5,17 @@
 //! Module for uvfits file format reading and writing
 //! Most of this was blatently stolen (with permission) from [Chris Jordan](https://github.com/cjordan)
 
-// use std::collections::HashSet;
-use std::path::Path;
-use std::{ffi::CString, ops::Range};
+use std::{
+    ffi::CString,
+    ops::Range,
+    path::{Path, PathBuf},
+};
 
 use fitsio::{errors::check_status as fits_check_status, FitsFile};
 use indicatif::ProgressStyle;
 use itertools::izip;
 use log::{trace, warn};
-use mwa_rust_core::{
+use marlu::{
     constants::VEL_C,
     erfa_sys,
     erfa_sys::ERFA_DJM0,
@@ -23,12 +25,16 @@ use mwa_rust_core::{
     precession::*,
     time, Jones, LatLngHeight, RADec, XyzGeodetic, ENH, UVW,
 };
-use ndarray::{Array3, ArrayView3, Axis};
 
-use crate::flags::flag_to_weight_array;
+use crate::{
+    flags::{flag_to_weight_array, get_weight_factor},
+    ndarray::{Array3, ArrayView3, Axis},
+};
 
-use super::error::{IOError, UvfitsWriteError};
-use super::WriteableVis;
+use super::{
+    error::{IOError, UvfitsWriteError},
+    WriteableVis,
+};
 
 /// From a `hifitime` [Epoch], get a formatted date string with the hours,
 /// minutes and seconds set to 0.
@@ -87,9 +93,9 @@ fn decode_uvfits_baseline(bl: usize) -> (usize, usize) {
 /// Note: only a single contiguous spectral window is supported.
 ///
 /// TODO: make writer and reader a single class?
-pub struct UvfitsWriter<'a> {
+pub struct UvfitsWriter {
     /// The path to the uvifts file.
-    path: &'a Path,
+    path: PathBuf,
 
     /// The number of uvfits rows. This is equal to `num_timesteps` *
     /// `num_baselines`.
@@ -113,12 +119,12 @@ pub struct UvfitsWriter<'a> {
     /// The RA/Dec where this observation is phased to
     phase_centre: RADec,
 
-    /// Array Position, geocentric
+    /// Array Position [Latitude (radians), Longitude (radians), Height (m)]
     array_pos: LatLngHeight,
 }
 
-impl<'a> UvfitsWriter<'a> {
-    /// Create a new uvfits file at the specified filename.
+impl UvfitsWriter {
+    /// Create a new uvfits file at the specified path.
     ///
     /// This will destroy any existing uvfits file at that path
     ///
@@ -128,7 +134,7 @@ impl<'a> UvfitsWriter<'a> {
     /// `start_epoch` can be calculated from GPS Time using the hifitime library, e.g.
     ///
     /// ```rust
-    /// use mwa_rust_core::time;
+    /// use marlu::time;
     /// let first_gps_time = 1196175296.0;
     /// let start_epoch = time::gps_to_epoch(first_gps_time);
     /// ```
@@ -149,13 +155,13 @@ impl<'a> UvfitsWriter<'a> {
     /// # Errors
     ///
     /// Will return an [`UvfitsWriteError`] if:
-    /// - there is an existing file at `filename` which cannot be removed.
+    /// - there is an existing file at `path` which cannot be removed.
     /// - a fits operation fails.
     ///
     /// TODO: replace all these args with birli_context
     #[allow(clippy::too_many_arguments)]
-    pub fn new(
-        filename: &'a Path,
+    pub fn new<T: AsRef<Path>>(
+        path: T,
         num_timesteps: usize,
         num_baselines: usize,
         num_chans: usize,
@@ -168,20 +174,23 @@ impl<'a> UvfitsWriter<'a> {
         array_pos: Option<LatLngHeight>,
     ) -> Result<Self, UvfitsWriteError> {
         // Delete any file that already exists.
-        if filename.exists() {
-            trace!("file {:?} exists, deleting", &filename);
-            std::fs::remove_file(&filename)?;
+        if path.as_ref().exists() {
+            trace!("file {:?} exists, deleting", &path.as_ref());
+            std::fs::remove_file(&path)?;
         }
 
         // Create a new fits file.
         let mut status = 0;
-        let c_filename = CString::new(filename.to_str().unwrap())?;
+        let c_path = CString::new(path.as_ref().to_str().unwrap())?;
         let mut fptr = std::ptr::null_mut();
-        trace!("initialising fits file with fitsio_sys ({:?})", &filename);
+        trace!(
+            "initialising fits file with fitsio_sys ({:?})",
+            &path.as_ref()
+        );
         unsafe {
             fitsio_sys::ffinit(
                 &mut fptr as *mut *mut _, /* O - FITS file pointer                   */
-                c_filename.as_ptr(),      /* I - name of file to create              */
+                c_path.as_ptr(),          /* I - name of file to create              */
                 &mut status,              /* IO - error status                       */
             );
         }
@@ -192,7 +201,7 @@ impl<'a> UvfitsWriter<'a> {
         let mut naxes = [0, 3, 4, num_chans as i64, 1, 1];
         let num_group_params = 5;
         let total_num_rows = num_timesteps * num_baselines;
-        trace!("setting group params in fits file ({:?})", &filename);
+        trace!("setting group params in fits file ({:?})", &path.as_ref());
         unsafe {
             fitsio_sys::ffphpr(
                 fptr,                  /* I - FITS file pointer                        */
@@ -209,15 +218,15 @@ impl<'a> UvfitsWriter<'a> {
         fits_check_status(status)?;
 
         // Finally close the file.
-        trace!("closing fits file ({:?})", &filename);
+        trace!("closing fits file ({:?})", &path.as_ref());
         unsafe {
             fitsio_sys::ffclos(fptr, &mut status);
         }
         fits_check_status(status)?;
 
         // Open the fits file with rust-fitsio.
-        trace!("opening  ({:?})", &filename);
-        let mut u = FitsFile::edit(&filename)?;
+        trace!("opening  ({:?})", &path.as_ref());
+        let mut u = FitsFile::edit(&path)?;
         let hdu = u.hdu(0)?;
         hdu.write_key(&mut u, "BSCALE", 1.0)?;
 
@@ -274,8 +283,6 @@ impl<'a> UvfitsWriter<'a> {
         hdu.write_key(&mut u, "TELESCOP", "MWA")?;
         hdu.write_key(&mut u, "INSTRUME", "MWA")?;
 
-        // TODO: write obsid?
-
         // This is apparently required...
         let history = CString::new("AIPS WTSCAL =  1.0").unwrap();
         unsafe {
@@ -323,7 +330,7 @@ impl<'a> UvfitsWriter<'a> {
         };
 
         Ok(Self {
-            path: filename,
+            path: path.as_ref().to_path_buf(),
             total_num_rows,
             current_num_rows: 0,
             centre_freq: centre_freq_hz,
@@ -333,7 +340,7 @@ impl<'a> UvfitsWriter<'a> {
         })
     }
 
-    /// Create a new uvfits file at the specified filename using an [`mwalib::CorrelatorContext`]
+    /// Create a new uvfits file at the specified path using an [`mwalib::CorrelatorContext`]
     ///
     /// # Details
     ///
@@ -343,8 +350,8 @@ impl<'a> UvfitsWriter<'a> {
     /// # Errors
     ///
     /// See: [`UvfitsWriter::new`]
-    pub fn from_mwalib(
-        filename: &'a Path,
+    pub fn from_mwalib<T: AsRef<Path>>(
+        path: T,
         context: &CorrelatorContext,
         mwalib_timestep_range: &Range<usize>,
         mwalib_coarse_chan_range: &Range<usize>,
@@ -366,8 +373,14 @@ impl<'a> UvfitsWriter<'a> {
         let centre_freq_hz =
             context.metafits_context.metafits_fine_chan_freqs_hz[img_centre_fine_chan_idx];
 
+        let obs_name = context.metafits_context.obs_name.clone();
+        let field_name = match obs_name.rsplit_once("_") {
+            Some((field_name, _)) => field_name,
+            None => obs_name.as_str(),
+        };
+
         Self::new(
-            filename,
+            path,
             mwalib_timestep_range.len(),
             mwalib_baseline_idxs.len(),
             num_img_chans,
@@ -376,7 +389,7 @@ impl<'a> UvfitsWriter<'a> {
             centre_freq_hz,
             centre_freq_chan,
             phase_centre,
-            Some(context.metafits_context.obs_name.as_str()),
+            Some(field_name),
             array_pos,
         )
     }
@@ -472,6 +485,9 @@ impl<'a> UvfitsWriter<'a> {
 
         hdu.write_key(&mut uvfits, "FREQ", self.centre_freq)?;
 
+        // Antenna position reference frame
+        hdu.write_key(&mut uvfits, "FRAME", "ITRF")?;
+
         // Get the Greenwich apparent sidereal time from ERFA.
         let mjd = self.start_epoch.as_mjd_utc_days();
         let gst = unsafe { erfa_sys::eraGst06a(ERFA_DJM0, mjd.floor(), ERFA_DJM0, mjd.floor()) }
@@ -487,13 +503,36 @@ impl<'a> UvfitsWriter<'a> {
         hdu.write_key(&mut uvfits, "UT1UTC", 0.0)?;
         hdu.write_key(&mut uvfits, "DATUTC", 0.0)?;
 
+        // AIPS 117 calls this TIMESYS, but Cotter calls in TIMSYS, so we do both.
         hdu.write_key(&mut uvfits, "TIMSYS", "UTC")?;
+        hdu.write_key(&mut uvfits, "TIMESYS", "UTC")?;
         hdu.write_key(&mut uvfits, "ARRNAM", "MWA")?;
         hdu.write_key(&mut uvfits, "NUMORB", 0)?; // number of orbital parameters in table
         hdu.write_key(&mut uvfits, "NOPCAL", 3)?; // Nr pol calibration values / IF(N_pcal)
         hdu.write_key(&mut uvfits, "FREQID", -1)?; // Frequency setup number
         hdu.write_key(&mut uvfits, "IATUTC", 33.0)?;
 
+        // -> EXTVER
+        // ---> in AIPS117:
+        // -----> on page 12, it's "Subarray number", type I
+        // -----> on page 84 onwards, all examples say "Version number of table"
+        // ---> in pyuvdata is 1, presumably since we're only writing a single
+        //   AIPS_AN version and someone assumed it was 1 indexed
+        // ---> @derwentx: I'm pretty sure the wrong description was pasted into
+        // EXTVER, and it's incorrectly being used as subarray number, when it
+        // should just be the version number of the table.
+        hdu.write_key(&mut uvfits, "EXTVER", 1)?;
+
+        // -> NO_IF - Number IFs (nIF)
+        // ---> in AIPS117: The value of the NO IF keyword shall specify the number of spectral
+        //  windows (IFs) in the data set. In the antenna file, this controls the dimension of the
+        //  polarization calibration value column.
+        // ---> in Cotter, this is not used.
+        // ---> since we can only deal with one spectral window at the moment,
+        //  this is fixed at 1, but this would change in
+        //  https://github.com/MWATelescope/Birli/issues/13
+        hdu.write_key(&mut uvfits, "NO_IF", 1)?;
+
         // Assume the station coordinates are "right handed".
         hdu.write_key(&mut uvfits, "XYZHAND", "RIGHT")?;
 
@@ -769,7 +808,8 @@ impl<'a> UvfitsWriter<'a> {
         mwalib_coarse_chan_range: &Range<usize>,
         mwalib_baseline_idxs: &[usize],
     ) -> Result<(), IOError> {
-        let weight_array = flag_to_weight_array(context, flag_array.view());
+        let weight_factor = get_weight_factor(context);
+        let weight_array = flag_to_weight_array(flag_array.view(), weight_factor);
         self.write_vis_mwalib(
             jones_array.view(),
             weight_array.view(),
@@ -786,7 +826,7 @@ impl<'a> UvfitsWriter<'a> {
 // - replace coarse chan range with...
 // - replace baseline indices with ...
 // - pass in tiles xyz geodetic
-impl<'a> WriteableVis for UvfitsWriter<'a> {
+impl WriteableVis for UvfitsWriter {
     fn write_vis_mwalib(
         &mut self,
         jones_array: ArrayView3<Jones<f32>>,
@@ -915,7 +955,7 @@ impl<'a> WriteableVis for UvfitsWriter<'a> {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use mwa_rust_core::time;
+    use marlu::time;
     use tempfile::NamedTempFile;
 
     #[test]
@@ -987,8 +1027,8 @@ mod tests {
 /// Tests which require the use of the aoflagger feature
 mod tests_aoflagger {
     use super::*;
-    use approx::assert_abs_diff_eq;
-    use mwa_rust_core::{
+    use marlu::{
+        approx::assert_abs_diff_eq,
         constants::{
             COTTER_MWA_HEIGHT_METRES, COTTER_MWA_LATITUDE_RADIANS, COTTER_MWA_LONGITUDE_RADIANS,
         },
@@ -1050,7 +1090,7 @@ mod tests_aoflagger {
         };
     }
 
-    macro_rules! assert_f64_string_keys_eq {
+    macro_rules! assert_f32_string_keys_eq {
         ($keys:expr, $left_fptr:expr, $left_hdu:expr, $right_fptr:expr, $right_hdu:expr) => {
             for key in $keys {
                 match (
@@ -1059,22 +1099,21 @@ mod tests_aoflagger {
                 ) {
                     (Ok(left_val), Ok(right_val)) => {
                         assert!(
-                            approx_eq!(f64, left_val, right_val, F64Margin::default()),
-                            "mismatch for short f64 key {}. {} != {} (margin={:?})",
+                            approx_eq!(f32, left_val, right_val, F32Margin::default()),
+                            "mismatch for short f32 key {}. {} != {}",
                             key,
                             left_val,
-                            right_val,
-                            F64Margin::default()
+                            right_val
                         );
                     }
                     (Err(err), Ok(right_val)) => {
                         panic!(
-                            "unable to get left short f64 key {}. Right val={}. err={}",
+                            "unable to get left short f32 key {}. Right val={}. err={}",
                             key, right_val, err
                         );
                     }
                     (.., Err(err)) => {
-                        panic!("unable to get right short f64 key {}. {}", key, err);
+                        panic!("unable to get right short f32 key {}. {}", key, err);
                     }
                 }
             }
@@ -1111,7 +1150,7 @@ mod tests_aoflagger {
                 _ => {
                     panic!("could not read left ant HDU as a table")
                 }
-            };
+            }
         };
     }
 
@@ -1250,7 +1289,7 @@ mod tests_aoflagger {
             _right_primary_hdu
         );
 
-        assert_f64_string_keys_eq!(
+        assert_f32_string_keys_eq!(
             vec![
                 "BITPIX", "NAXIS", "NAXIS1", "NAXIS2", "NAXIS3", "NAXIS4", "NAXIS5", "NAXIS6",
                 "BSCALE", "PSCAL1", "PZERO1", "PSCAL2", "PZERO2", "PSCAL3", "PZERO3", "PSCAL4",
@@ -1259,8 +1298,11 @@ mod tests_aoflagger {
                 // This is actually incorrect in Cotter, see https://github.com/MWATelescope/Birli/issues/6
                 // "CRVAL4",
                 "CDELT4", "CRPIX4", "CRVAL5", "CRPIX5", "CDELT5", "CRVAL6", "CRPIX6", "CDELT6",
-                "EPOCH", "OBSRA", "OBSDEC",
-                // TODO: "FIBRFACT", (v-factor, cable type?)
+                "EPOCH", "OBSRA",
+                "OBSDEC",
+                // "FIBRFACT"
+                // the velocity factor of electic fields in RG-6 like coax.
+                // this is incorrect in Cotter. It writes 2.0 instead of 1.204
             ],
             left_fptr,
             _left_primary_hdu,
@@ -1278,7 +1320,7 @@ mod tests_aoflagger {
                 "XTENSION", "TTYPE1", "TFORM1", "TTYPE2", "TFORM2", "TUNIT2", "TTYPE3", "TFORM3",
                 "TTYPE4", "TFORM4", "TTYPE5", "TFORM5", "TUNIT5", "TTYPE6", "TFORM6", "TTYPE7",
                 "TFORM7", "TUNIT7", "TTYPE8", "TFORM8", "TTYPE9", "TFORM9", "TTYPE10", "TFORM10",
-                "TUNIT10", "TTYPE11", "TFORM11", "EXTNAME", "TIMSYS", "ARRNAM",
+                "TUNIT10", "TTYPE11", "TFORM11", "EXTNAME", "TIMSYS", "ARRNAM", "RDATE"
             ],
             left_fptr,
             _left_ant_hdu,
@@ -1286,15 +1328,14 @@ mod tests_aoflagger {
             _right_ant_hdu
         );
 
-        assert_f64_string_keys_eq!(
+        assert_f32_string_keys_eq!(
             vec![
                 "BITPIX", "NAXIS", "NAXIS1", "NAXIS2", "PCOUNT", "GCOUNT", "TFIELDS", //
-                // TODO: "ARRAYX", "ARRAYY", "ARRAYZ",
+                "ARRAYX", "ARRAYY", "ARRAYZ",
                 // "FREQ", // This is incorrect in Cotter. See: https://github.com/MWATelescope/Birli/issues/6
-                // TODO: "GSTIA0",
-                "DEGPDY", //
-                // TODO: "RDATE",
-                "POLARX", "POLARY", "UT1UTC", "DATUTC", "NUMORB", "NOPCAL", "FREQID", "IATUTC",
+                // "GSTIA0", // TODO: this is off from Cotter by about 5e-2
+                "DEGPDY", "POLARX", "POLARY", "UT1UTC", "DATUTC", "NUMORB", "NOPCAL", "FREQID",
+                "IATUTC",
             ],
             left_fptr,
             _left_ant_hdu,
@@ -1611,11 +1652,11 @@ mod tests_aoflagger {
             Some(flag_array),
         );
 
-        let strategy_filename = &aoflagger.FindStrategyFileMWA();
+        let strategy_path = &aoflagger.FindStrategyFileMWA();
 
         let flag_array = flag_jones_array_existing(
             &aoflagger,
-            strategy_filename,
+            strategy_path,
             &jones_array,
             Some(flag_array),
             true,
@@ -1705,13 +1746,276 @@ mod tests_aoflagger {
 
         let mut birli_fptr = fits_open!(&tmp_uvfits_file.path()).unwrap();
 
+        let expected_center_freq = 229760000.;
+
         let birli_vis_hdu = fits_open_hdu!(&mut birli_fptr, 0).unwrap();
         let birli_vis_freq: f64 =
             get_required_fits_key!(&mut birli_fptr, &birli_vis_hdu, "CRVAL4").unwrap();
-        assert_abs_diff_eq!(birli_vis_freq, 229760000.);
+        assert_abs_diff_eq!(birli_vis_freq, expected_center_freq);
         let birli_ant_hdu = fits_open_hdu!(&mut birli_fptr, 1).unwrap();
         let birli_ant_freq: f64 =
             get_required_fits_key!(&mut birli_fptr, &birli_ant_hdu, "FREQ").unwrap();
-        assert_abs_diff_eq!(birli_ant_freq, 229760000.);
+        assert_abs_diff_eq!(birli_ant_freq, expected_center_freq);
+    }
+
+    /// Tests for AIPS 117 compliance.
+    /// See https://github.com/MWATelescope/Birli/issues/9
+    /// and ftp://ftp.aoc.nrao.edu/pub/software/aips/TEXT/PUBL/AIPSMEM117.PS
+    #[test]
+    fn aips_117() {
+        let context = get_mwa_ord_context();
+
+        let tmp_uvfits_file = NamedTempFile::new().unwrap();
+
+        let img_timestep_idxs = get_flaggable_timesteps(&context).unwrap();
+        assert_eq!(img_timestep_idxs.len(), 4);
+        let img_timestep_range =
+            *img_timestep_idxs.first().unwrap()..(*img_timestep_idxs.last().unwrap() + 1);
+        let img_coarse_chan_idxs = &context.common_coarse_chan_indices;
+        let img_coarse_chan_range =
+            *img_coarse_chan_idxs.first().unwrap()..(*img_coarse_chan_idxs.last().unwrap() + 1);
+        let img_baseline_idxs = (0..context.metafits_context.num_baselines).collect::<Vec<_>>();
+
+        let array_pos = Some(LatLngHeight {
+            longitude_rad: COTTER_MWA_LONGITUDE_RADIANS,
+            latitude_rad: COTTER_MWA_LATITUDE_RADIANS,
+            height_metres: COTTER_MWA_HEIGHT_METRES,
+        });
+
+        let mut u = UvfitsWriter::from_mwalib(
+            tmp_uvfits_file.path(),
+            &context,
+            &img_timestep_range,
+            &img_coarse_chan_range,
+            &img_baseline_idxs,
+            array_pos,
+        )
+        .unwrap();
+
+        let flag_array = init_flag_array(
+            &context,
+            &img_timestep_range,
+            &img_coarse_chan_range,
+            Some(get_antenna_flags(&context)),
+        );
+
+        let (jones_array, flag_array) = context_to_jones_array(
+            &context,
+            &img_timestep_range,
+            &img_coarse_chan_range,
+            Some(flag_array),
+        );
+
+        u.write_jones_flags(
+            &context,
+            &jones_array,
+            &flag_array,
+            &img_timestep_range,
+            &img_coarse_chan_range,
+            &img_baseline_idxs,
+        )
+        .unwrap();
+
+        u.write_ants_from_mwalib(&context.metafits_context).unwrap();
+
+        let mut birli_fptr = fits_open!(&tmp_uvfits_file.path()).unwrap();
+
+        // /////////// //
+        // PRIMARY HDU //
+        // /////////// //
+
+        let birli_vis_hdu = fits_open_hdu!(&mut birli_fptr, 0).unwrap();
+
+        // -> OBJECT
+        let birli_vis_object: String =
+            get_required_fits_key!(&mut birli_fptr, &birli_vis_hdu, "OBJECT").unwrap();
+        assert_eq!(birli_vis_object, "ForA");
+        // -> TELESCOP
+        let birli_vis_telescop: String =
+            get_required_fits_key!(&mut birli_fptr, &birli_vis_hdu, "TELESCOP").unwrap();
+        assert_eq!(birli_vis_telescop, "MWA");
+        // -> INSTRUME
+        let birli_vis_instrume: String =
+            get_required_fits_key!(&mut birli_fptr, &birli_vis_hdu, "INSTRUME").unwrap();
+        assert_eq!(birli_vis_instrume, "MWA");
+        // -> DATE-OBS
+        // ---> in AIPS117:
+        // -----> on page 12: "The value of the RDATE parameter will be the date for which the time
+        //  system parameters GSTIA0, DECPDY, and IATUTC apply.
+        //  If the table contains orbital parameters for orbiting antennæ, this keyword also
+        //  designates the epoch for the orbital parameters.
+        //  (This is copy-pasted twice)
+        // -----> on page 85 onwards, all examples show YYYY-MM-DD format
+        // ---> in Cotter, it is given in ISO8601 (YYYY-MM-DDTHH:mm:ss) with time fixed to 00:00:00.
+        // TODO: determine whether this field should have the time.
+        // let birli_vis_date_obs: String =
+        // get_required_fits_key!(&mut birli_fptr, &birli_vis_hdu, "DATE-OBS").unwrap();
+        // assert_eq!(birli_vis_date_obs, "2017-12-01T14:54:38");
+
+        // -> DATE-MAP - File processing date
+        // ---> not in Cotter, not mandatory, so not written
+        // -> BSCALE
+        let birli_ant_bscale: f32 =
+            get_required_fits_key!(&mut birli_fptr, &birli_vis_hdu, "BSCALE").unwrap();
+        assert_abs_diff_eq!(birli_ant_bscale, 1.);
+        // -> BUNIT - units,
+        // ---> not in Cotter, not mandatory, so not written
+        // let birli_ant_bunit: String =
+        // get_required_fits_key!(&mut birli_fptr, &birli_vis_hdu, "BUNIT").unwrap();
+        // -> EQUINOX - Equinox of source coordinates and uvw
+        // ---> not in Cotter, not mandatory, so not written
+        // -> ALTRPIX - Reference pixel for velocity
+        // ---> not in Cotter, not mandatory, so not written
+
+        // /////////// //
+        // ANTENNA HDU //
+        // /////////// //
+
+        let birli_ant_hdu = fits_open_hdu!(&mut birli_fptr, 1).unwrap();
+
+        // -> EXTVER
+        // ---> in AIPS117:
+        // -----> on page 12, it's "Subarray number", type I
+        // -----> on page 84 onwards, all examples say "Version number of table"
+        // ---> in pyuvdata is 1, presumably since we're only writing a single
+        //   AIPS_AN version and someone assumed it was 1 indexed
+        // ---> @derwentx: I'm pretty sure the wrong description was pasted into
+        // EXTVER, and it's incorrectly being used as subarray number, when it
+        // should just be the version number of the table.
+        let birli_ant_extver: i32 =
+            get_required_fits_key!(&mut birli_fptr, &birli_ant_hdu, "EXTVER").unwrap();
+        assert_eq!(birli_ant_extver, 1);
+        // -> ARRAY{X|Y|Z} = coordinate of array center (meters)
+        let birli_ant_arrayx: f32 =
+            get_required_fits_key!(&mut birli_fptr, &birli_ant_hdu, "ARRAYX").unwrap();
+        assert_abs_diff_eq!(birli_ant_arrayx, -2_559_453.3, epsilon = f32::EPSILON);
+        let birli_ant_arrayy: f32 =
+            get_required_fits_key!(&mut birli_fptr, &birli_ant_hdu, "ARRAYY").unwrap();
+        assert_abs_diff_eq!(birli_ant_arrayy, 5_095_371.5, epsilon = f32::EPSILON);
+        let birli_ant_arrayz: f32 =
+            get_required_fits_key!(&mut birli_fptr, &birli_ant_hdu, "ARRAYZ").unwrap();
+        assert_abs_diff_eq!(birli_ant_arrayz, -2_849_056.8, epsilon = f32::EPSILON);
+        // -> GSTIAO = GST at 0h on reference date (degrees)
+        // ---> our value is out from cotter's by about 5e-2. Not sure if this is an issue.
+        // ---> notes from @mkolopanis: this one depends on your RDATE but I'm not 100% sure what
+        //   sets the RDATE for our kind of data. We just use 0 to spoof this because we don't
+        //   directly use it in any of our calculations.
+        let birli_ant_gstia0: f32 =
+            get_required_fits_key!(&mut birli_fptr, &birli_ant_hdu, "GSTIA0").unwrap();
+        assert_abs_diff_eq!(birli_ant_gstia0, 70.044_16, epsilon = 5e-2);
+        // -> DEGPDY = Earth’s rotation rate (degrees/day)
+        let birli_ant_degpdy: f32 =
+            get_required_fits_key!(&mut birli_fptr, &birli_ant_hdu, "DEGPDY").unwrap();
+        assert_abs_diff_eq!(birli_ant_degpdy, 360.985, epsilon = f32::EPSILON);
+        // -> FREQ = Reference frequency (Hz)
+        // ---> Cotter-calculated value was slightly incorrect because of
+        //   https://github.com/MWATelescope/Birli/issues/6
+        let birli_ant_freq: f32 =
+            get_required_fits_key!(&mut birli_fptr, &birli_ant_hdu, "FREQ").unwrap();
+        assert_abs_diff_eq!(birli_ant_freq, 229760000., epsilon = f32::EPSILON);
+        // -> RDATE = Reference date
+        // ---> in AIPS117:
+        // -----> on page 12: "The value of the RDATE parameter will be the date for which the time
+        //  system parameters GSTIA0, DECPDY, and IATUTC apply.
+        //  If the table contains orbital parameters for orbiting antennæ, this keyword also
+        //  designates the epoch for the orbital parameters.
+        //  (This is copy-pasted twice)
+        // -----> on page 85 onwards, all examples show YYYY-MM-DD format
+        // ---> in Cotter, it is given in ISO8601 (YYYY-MM-DDTHH:mm:ss) with time fixed to 00:00:00.
+        let birli_ant_rdate: String =
+            get_required_fits_key!(&mut birli_fptr, &birli_ant_hdu, "RDATE").unwrap();
+        assert_eq!(birli_ant_rdate, "2017-12-01T00:00:00.0");
+        // -> POLAR{X|Y} = coordinate of North Pole (arc seconds)
+        // ---> notes from @mkolopanis: 0 makes sense here I think. Unless there's an offset from
+        //   North Pole in images.
+        let birli_ant_polarx: f32 =
+            get_required_fits_key!(&mut birli_fptr, &birli_ant_hdu, "POLARX").unwrap();
+        assert_abs_diff_eq!(birli_ant_polarx, 0., epsilon = f32::EPSILON);
+        let birli_ant_polary: f32 =
+            get_required_fits_key!(&mut birli_fptr, &birli_ant_hdu, "POLARX").unwrap();
+        assert_abs_diff_eq!(birli_ant_polary, 0., epsilon = f32::EPSILON);
+        // -> UT1UTC = UT1 - UTC (sec)
+        // ---> notes from @mkolopanis: we also use 0 here.
+        let birli_ant_ut1utc: f32 =
+            get_required_fits_key!(&mut birli_fptr, &birli_ant_hdu, "UT1UTC").unwrap();
+        assert_abs_diff_eq!(birli_ant_ut1utc, 0., epsilon = f32::EPSILON);
+        // -> DATUTC = "time system - UTC (sec)" (huh)
+        // ---> notes from @mkolopanis: would 0 if your TIMESYS is UTC. We assume UTC ourselves so
+        //   it is always 0
+        let birli_ant_datutc: f32 =
+            get_required_fits_key!(&mut birli_fptr, &birli_ant_hdu, "DATUTC").unwrap();
+        assert_abs_diff_eq!(birli_ant_datutc, 0., epsilon = f32::EPSILON);
+
+        // -> TIMSYS/TIMESYS "Time system"
+        // ---> in AIPS117:
+        // -----> on page 12 it's listed as `TIMESYS`, type A in Mandatory keywords
+        // -----> on page 13 it's Time system. The TIMSYS keyword shall specify the time system used
+        //        for the array. It shall either have the value ’IAT’, denoting international atomic
+        //        time, or the value ’UTC’, denoting coordinated universal time. This indicates
+        //        whether the zero hour for the TIME parameter in the UV DATA table is midnight IAT
+        //        or midnight UTC.
+        // -----> on page 87, it's `TIMSYS` in an example
+        // ---> in CASA it's `TIMSYS`
+        // ---> in pyuvdata: they look for both
+        // ---> in Cotter it's `TIMSYS`
+        // So ¯\_(ツ)_/¯
+        let birli_ant_timesys: String =
+            get_required_fits_key!(&mut birli_fptr, &birli_ant_hdu, "TIMESYS").unwrap();
+        assert_eq!(birli_ant_timesys, "UTC");
+        let birli_ant_timsys: String =
+            get_required_fits_key!(&mut birli_fptr, &birli_ant_hdu, "TIMSYS").unwrap();
+        assert_eq!(birli_ant_timsys, "UTC");
+        // -> ARRNAM
+        let birli_ant_timsys: String =
+            get_required_fits_key!(&mut birli_fptr, &birli_ant_hdu, "ARRNAM").unwrap();
+        assert_eq!(birli_ant_timsys, "MWA");
+        // -> XYZHAND
+        // ---> Birli assumes the station coordinates are "right handed".
+        let birli_ant_frame: String =
+            get_required_fits_key!(&mut birli_fptr, &birli_ant_hdu, "XYZHAND").unwrap();
+        assert_eq!(birli_ant_frame, "RIGHT");
+        // -> FRAME - Coordinate frame
+        // ---> in AIPS117: The value of the FRAME keyword shall be a string that identifies the
+        //  coordinate system used for antenna coordinates. At present, only one value of the FRAME
+        //  keyword has been defined (’ITRF’), although ’????’ is widely used to reflect ignorance.
+        // ---> notes from @mkolopanis: the "FRAME" keyword in particular in the "AIPS AN" table is
+        //   important to know which frame the antenna positions are recorded in.
+        let birli_ant_frame: String =
+            get_required_fits_key!(&mut birli_fptr, &birli_ant_hdu, "FRAME").unwrap();
+        assert_eq!(birli_ant_frame, "ITRF");
+        // -> NUMORB - Number orbital parameters in table (norb)
+        let birli_ant_numorb: i32 =
+            get_required_fits_key!(&mut birli_fptr, &birli_ant_hdu, "NUMORB").unwrap();
+        assert_eq!(birli_ant_numorb, 0);
+        // -> NO_IF - Number IFs (nIF)
+        // ---> in AIPS117: The value of the NO IF keyword shall specify the number of spectral
+        //  windows (IFs) in the data set. In the antenna file, this controls the dimension of the
+        //  polarization calibration value column.
+        // ---> in Cotter, this is not used.
+        // ---> notes from @mkolopanis: we handle NO_IF as the number of independent spectral
+        //  windows. I can see how you could interpret each coarse band as its own spectral window.
+        // We just view the full band as a single window since it is contiguous with all coarse
+        // bands present. The idea of multiple spectral windows I think is more common for things
+        // like the VLA when you observe in multiple bands that do not form a contiguous frequency
+        // band if concatenated.
+        let birli_ant_no_if: i32 =
+            get_required_fits_key!(&mut birli_fptr, &birli_ant_hdu, "NO_IF").unwrap();
+        assert_eq!(birli_ant_no_if, 1);
+        // -> NOPCAL - Number of polarization calibration values / IF (npcal)
+        let birli_ant_nopcal: i32 =
+            get_required_fits_key!(&mut birli_fptr, &birli_ant_hdu, "NOPCAL").unwrap();
+        assert_eq!(birli_ant_nopcal, 3);
+        // -> POLTYPE - Type of polarization calibration
+        // ---> If the table contains information about the polarization characteristics of
+        //  the feeds, then the feed parametrization that is used shall be indicated by the value of
+        //  the POLTYPE keyword, as given in Table 9.
+        // ---> not used.
+        // let birli_ant_poltype: i32 =
+        //     get_required_fits_key!(&mut birli_fptr, &birli_ant_hdu, "POLTYPE").unwrap();
+
+        // -> FREQID - Frequency setup number
+        let birli_ant_freqid: i32 =
+            get_required_fits_key!(&mut birli_fptr, &birli_ant_hdu, "FREQID").unwrap();
+        assert_eq!(birli_ant_freqid, -1);
     }
 }
